<?php
namespace Modules\Parametro\Controllers; 

use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DTO\Response\Error;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DataSource\ParameterBag;
use Zeedhi\Framework\Controller;
use Zeedhi\Framework\Controller\Crud;
use Zeedhi\Framework\DataSource;
use Zeedhi\Framework\DTO\Response\Notification;


class Gop00200ControleUnidadeGerenteController extends \Teknisa\GER\Controllers\TeknisaCrud {
    
    protected $crudService;
    protected $connection;
    protected $newCode;
    protected $environment;
    
    public function __construct($crudService, ParameterBag $parameterBag, $dataSourceManager, $connection, $newCode, $environment) {
        $this->dataSourceManager    = $dataSourceManager;
        $this->connection    		= $connection;
        $this->newCode              = $newCode;
        $this->environment          = $environment;
        parent::__construct($crudService, $parameterBag);
    }
    

    public function excluirGrupoGerente(DTO\Request $request, DTO\Response $response ){
        $rows = $request->getDataSet()->getRows();

        foreach($rows as $row){
            try{
                
                $this->connection->beginTransaction();
                
                $conditions = array (
        			array (
            				"columnName"    =>"CDGRUPGERENTEOP",
            				"operator"      =>"=",
            				"value"         => $row["CDGRUPGERENTEOP"]
            			)
        		);
        		
        		$operadores = $this->crudService->getFilteredDataSet('opgrupgerenteop', $conditions)->getRows();

        		if(!empty($operadores)){
        		    $menssagem = array();
        		    array_push($menssagem, "Não é possível excluir grupo com gerentes cadastrados!");
        		    $response->addDataSet(new DataSet("errorList", $menssagem));
        		    return 0;
        		}else if(empty($operadores)){

            		$filiais = $this->crudService->getFilteredDataSet('filialoperop', $conditions);
            		$this->dataSourceManager->delete($filiais);
            		
            		$grupo = $this->crudService->getFilteredDataSet('grupgerenteop', $conditions);
            		$this->dataSourceManager->delete($grupo);
            		
            		$this->connection->commit();
        		}

        		
            } catch (\Exception $e) {
    			$this->connection->rollback();
                $response->setError(new DTO\Response\Error($e->getMessage(), 1));
            }
            
        }
    }
    
    public function saveOperadorGrupo(DTO\Request $request, DTO\Response $response ){
        $row = $request->getRow();
        try{
            if(empty($row))
                throw new \Exception("Nao foi possivel realizar a operação!");
                
            $validaOperador = $this->validaOperadorGrupo($row["CDOPERADOR"])->getRows();
            if(!empty($validaOperador)){
                if ($row['CDGRUPGERENTEOP'] == $validaOperador[0]['CDGRUPGERENTEOP']){
                    throw new \Exception("Operador vinculado ao grupo atual! ");
                }else {
                    throw new \Exception("Operador vinculado em outro grupo!");
                }
            }
                
            $newCode = array(
                            'CDOPERADOR' => $row["CDOPERADOR"],
                            'CDGRUPGERENTEOP' => $row["CDGRUPGERENTEOP"],
                            '__is_new' => true
                        );
            
            $rowInsert = new DTO\Request\Row($newCode, '', '', '');
           
            $rowToSave =    $this->crudService->getRowsToSave($rowInsert, "opgrupgerenteop"); 
            $this->crudService->save("opgrupgerenteop", $rowToSave);
            
        } catch (\Exception $e) {
            
            $response->setError(new DTO\Response\Error($e->getMessage(), 1));
        }
    }
    
    public function excluirOperadorGrupo(DTO\Request $request, DTO\Response $response ){
        $rows = $request->getDataSet()->getRows();

        foreach($rows as $row){
            try{
                
                $this->connection->beginTransaction();
                
                $conditions = array (
        			array (
            				"columnName"    =>"CDGRUPGERENTEOP",
            				"operator"      =>"=",
            				"value"         => $row["CDGRUPGERENTEOP"]
            			),
            		array (
            				"columnName"    =>"CDOPERADOR",
            				"operator"      =>"=",
            				"value"         => $row["CDOPERADOR"]
            			)
        		);
        		
        		$operadores = $this->crudService->getFilteredDataSet('opgrupgerenteop', $conditions);
                $this->dataSourceManager->delete($operadores);
        		
        		$filiais = $this->crudService->getFilteredDataSet('filialoperop', $conditions);
        		$this->dataSourceManager->delete($filiais);

        		$this->connection->commit();
        		

            } catch (\Exception $e) {
    			$this->connection->rollback();
                $response->setError(new DTO\Response\Error($e->getMessage(), 1));
            }
            
        }
    }
    
    public function validaOperadorGrupo($cdOperador){
        $params = array (
            array (
                "columnName"    =>"CDOPERADOR",
                "operator"      =>"=",
                "value"         => $cdOperador
            )
        );
		$dataSet = $this->crudService->getFilteredDataSet('opgrupgerenteop', $params);
		return $dataSet;
    }
    
    public function atualizaFilialGrupo(DTO\Request $request, DTO\Response $response ){
        $rows = $request->getRow(); 
        $this->connection->beginTransaction();
        try {
            $params = array(
                "P_CDFILIAL"  => $rows["CDFILIAL"],
                "P_CDGRUPGERENTEOP"  => $rows["CDGRUPGERENTEOP"],
                "P_CDOPERADOR"  => $rows["CDOPERADOR"],
                "P_IDACESSOCALC"  => $rows["IDACESSOCALC"], 
                "P_IDACESSOAPROV"  => $rows["IDACESSOAPROV"]
            );
            
            $query = "UPDATE FILIALOPEROP 
                         SET IDACESSOCALC = :P_IDACESSOCALC, IDACESSOAPROV = :P_IDACESSOAPROV
                       WHERE CDFILIAL = :P_CDFILIAL AND CDGRUPGERENTEOP = :P_CDGRUPGERENTEOP AND CDOPERADOR = :P_CDOPERADOR";
                       
            $stmt = $this->connection->prepare($query);
            $stmt->execute($params);
            
            $this->connection->commit();
        } catch (Exception $e){
            $this->connection->rollback();
            $response->setError(new Error($e->getMessage()));
        } 
    }
    
    public function saveFilialGrupo(DTO\Request $request, DTO\Response $response ){
        $rows = $request->getRow();
        $this->connection->beginTransaction();
        try {
            foreach($rows['CDFILIAL'] as $filial){
                $params = [
                    'P_CDFILIAL'  => $filial,
                    'P_CDGRUPGERENTEOP'  => $rows['CDGRUPGERENTEOP'],
                    'P_CDOPERADOR'  => $rows['CDOPERADOR'],
                    'P_IDACESSOCALC'  => $rows['IDACESSOCALC'], 
                    'P_IDACESSOAPROV'  => $rows['IDACESSOAPROV'],
                    'P_NRORG'     => $this->environment->getNrOrg()
                ];
                
                $query = "INSERT INTO FILIALOPEROP (CDFILIAL, CDGRUPGERENTEOP, CDOPERADOR, IDACESSOCALC, IDACESSOAPROV, NRORG) 
                                            VALUES (:P_CDFILIAL,:P_CDGRUPGERENTEOP, :P_CDOPERADOR, :P_IDACESSOCALC, :P_IDACESSOAPROV, :P_NRORG )";
                $stmt = $this->connection->prepare($query);
                $stmt->execute($params);
            }
            $this->connection->commit();
        } catch (Exception $e){
            $this->connection->rollback();
            $response->setError(new Error($e->getMessage()));
        } 
    }
    
    public function criarListaValores($lista){
        $this->connection->beginTransaction();
        $nrlista = $this->newCode->getCodeByTableName('LISTA', $this->environment->getNrorgTrab(), 1, 10, false);
        try{
            foreach($lista as $value){
                $this->insertLista($nrlista, $value);
            }
            $this->connection->commit();
            return $nrlista;
        } catch(\Exception $e){
            $this->connection->rollback();
            throw $e;
        }
    }
    
    public function insertLista($nrlista, $cdlista) {
        $params = array(
            "NRLISTA" => $nrlista,
            "CDLISTA" => $cdlista
        );
        
        $sql = "INSERT INTO LISTA (NRLISTA, CDLISTA) VALUES (:NRLISTA, :CDLISTA)";
        $query = $this->connection->prepare($sql);
        $query->execute($params);
    }
    
    public function saveReplicacaoGerente(DTO\Request $request, DTO\Response $response ){
        $rows = $request->getRow();
        try {
            $listOperador = $this->criarListaValores($rows['GERENTEDESTINO']);
            $this->connection->beginTransaction();
            $params = [
                'P_CDGRUPGERENTEOP'  => $rows['CDGRUPGERENTEOP'],
                'P_CDOPERORIGEM'  => $rows['CDOPERADOR'],
                'P_CDLISTAOPER'  => $listOperador,
                'P_NRORG'     => $this->environment->getNrOrg()
            ];
            $query = " BEGIN          
                            FOR C_LISTA IN (
                                SELECT
                                         LA.CDLISTA LACDOPERADOR,
                                        :P_NRORG NRORG
                                  FROM LISTA LA
                                 WHERE LA.NRLISTA = :P_CDLISTAOPER
                                )
                         LOOP 
                                INSERT INTO OPGRUPGERENTEOP OP (
                                    CDGRUPGERENTEOP,
                                    CDOPERADOR,
                                    NRORG
                                )          
                                SELECT
                                       :P_CDGRUPGERENTEOP, 
                                       C_LISTA.LACDOPERADOR,
                                       :P_NRORG
                                  FROM DUAL 
                                 WHERE NOT EXISTS (
                                    SELECT  
                                      OP.NRORG
                                      FROM OPGRUPGERENTEOP OP
                                     WHERE OP.CDOPERADOR = C_LISTA.LACDOPERADOR
                                       AND OP.CDGRUPGERENTEOP = :P_CDGRUPGERENTEOP
                                       AND OP.NRORG = :P_NRORG
                                );
                                DELETE 
                                  FROM FILIALOPEROP
                                 WHERE   
                                        CDOPERADOR = C_LISTA.LACDOPERADOR
                                        AND CDGRUPGERENTEOP = :P_CDGRUPGERENTEOP
                                        AND NRORG = :P_NRORG;
                                
                                INSERT INTO FILIALOPEROP (
                                      CDFILIAL,
                                      CDGRUPGERENTEOP,
                                      CDOPERADOR,
                                      IDACESSOCALC,
                                      IDACESSOAPROV,
                                      NRORG
                                )
                                SELECT 
                                       CDFILIAL,
                                       CDGRUPGERENTEOP,
                                       C_LISTA.LACDOPERADOR,
                                       IDACESSOCALC,
                                       IDACESSOAPROV,
                                       NRORG
                                  FROM FILIALOPEROP
                                 WHERE CDGRUPGERENTEOP = :P_CDGRUPGERENTEOP 
                                   AND CDOPERADOR = :P_CDOPERORIGEM;
                            
                              END LOOP;
                        END;";
            $stmt = $this->connection->prepare($query);
            $stmt->execute($params);
            $this->connection->commit();
        } catch (Exception $e){
            $this->connection->rollback();
            $response->setError(new Error($e->getMessage()));
        } 
    }
}